/*******************************************************************************

    This file is part of mDictionary.

    mDictionary is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    mDictionary is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with mDictionary.  If not, see <http://www.gnu.org/licenses/>.

    Copyright 2010 Comarch S.A.

*******************************************************************************/

/*! \file MainWidget.cpp
    \brief Implements widget for maemo that allows user to search quickly right from home screen using mdictionary.

    \author Mateusz Półrola <mateusz.polrola@comarch.pl>
*/

#include "MainWidget.h"

#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <QtGui/QX11Info>
#include <QtDBus/QDBusMessage>
#include <QtDBus/QDBusPendingCall>
#include <QList>
#include <QDebug>


HomeWidget::HomeWidget(QWidget *parent):
    QWidget(parent)
{
    //setting application name, it will use it later to set home applet id
    QCoreApplication::instance()->setApplicationName(
            "mDictionary desktop widget");

    //setting this attribute to get transparent backgorund
    setAttribute(Qt::WA_TranslucentBackground);

    initializeX11();

    initializeUI();
}


void HomeWidget::initializeUI() {


    horizontalLayout = new QHBoxLayout;
    setLayout(horizontalLayout);


    searchStringLineEdit = new QLineEdit;

    searchButton = new QPushButton(tr("Search"));
    searchButton->setMaximumHeight(60);

    logo = new QToolButton();
    logo->setIcon(QIcon(":/icons/64x64/mdictionary.png"));

    horizontalLayout->addWidget(logo);
    horizontalLayout->addWidget(searchStringLineEdit);
    horizontalLayout->addWidget(searchButton);
    horizontalLayout->setContentsMargins(10,10,10,10);

    connect(searchStringLineEdit, SIGNAL(returnPressed()),
            this, SLOT(search()));

    connect(searchButton, SIGNAL(clicked()),
            this, SLOT(search()));

    connect(logo, SIGNAL(clicked()),
            this, SLOT(logoClicked()));


    setMinimumHeight(100);
    setMaximumHeight(100);
}

void HomeWidget::initializeX11() {

    Atom winTypeAtom = XInternAtom(QX11Info::display(),
                                   "_NET_WM_WINDOW_TYPE",
                                   false);

    Atom homeAppletAtom = XInternAtom(QX11Info::display(),
                                     "_HILDON_WM_WINDOW_TYPE_HOME_APPLET",
                                     false);

    Atom appletIDAtom = XInternAtom(QX11Info::display(),
                                    "_HILDON_APPLET_ID",
                                    false);

    Atom utf8Atom = XInternAtom(QX11Info::display(),
                                "UTF8_STRING",
                                false);


    // Set window type to home applet
    XChangeProperty(QX11Info::display(), winId(), winTypeAtom,
                    XA_ATOM, 32, PropModeReplace,
                    (unsigned char *) &homeAppletAtom, 1);

    // Applet id will be the same as application name
    QByteArray id (
            QCoreApplication::instance()->applicationName().remove(' ').toUtf8());


    XChangeProperty(QX11Info::display(), winId(), appletIDAtom, utf8Atom, 8,
                    PropModeReplace, (unsigned char *)id.constData(),
                    id.length());
}

void HomeWidget::search() {
    if(searchStringLineEdit->text().isEmpty()) return;

    QDBusMessage message =
            QDBusMessage::createMethodCall("com.comarch.mdictionary",
                                           "/mainWindow",
                                           "com.comarch.mdictionary",
                                           "search");

    QList<QVariant> args;
    args.append(searchStringLineEdit->text());
    message.setArguments(args);

    QDBusConnection::sessionBus().send(message);

    searchStringLineEdit->clear();
}

void HomeWidget::logoClicked() {
    QDBusMessage message =
            QDBusMessage::createMethodCall("com.comarch.mdictionary",
                                           "/mainWindow",
                                           "com.comarch.mdictionary",
                                           "showApplication");
    QDBusConnection::sessionBus().send(message);
}



void HomeWidget::paintEvent(QPaintEvent *event) {
    //custom painting of rounded corners
    QPainter p(this);

    p.setBrush(palette().background());

    p.setPen(Qt::NoPen);

    p.drawRoundedRect(rect(), 25, 25);

    p.end();

    QWidget::paintEvent(event);
}
