/*******************************************************************************

    This file is part of mDictionary.

    mDictionary is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    mDictionary is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with mDictionary.  If not, see <http://www.gnu.org/licenses/>.

    Copyright 2010 Comarch S.A.

*******************************************************************************/

/*! \file History.cpp
    \brief Object with search history

    \author Mateusz Półrola <mateusz.polrola@comarch.pl>
*/

#include "../../include/History.h"
#include <QDebug>

History::History(int maxSize, QObject *parent) :
    QObject(parent)
{
    _maxSize = maxSize;
    currentElement = -1;
    _prevAvailable = false;
    _nextAvailable = false;
    _listAvailable = false;
}

void History::setMaxSize(int size) {
    if(maxSize() <= 0) return;
    if(size > _maxSize) {
        _maxSize = size;
    }
    else {
        _maxSize = size;
        if(currentElement > 0) {
            _history.remove(0, currentElement);
        }

        while(_history.size() > _maxSize) {
            _history.pop_back();
        }

        setCurrentElement(0);
    }
}

int History::maxSize() {
    return _maxSize;
}

void History::add(QString word) {
    if(currentElement != -1) {
        //we search for the same word so we don't add it again
        if(_history[currentElement] == word)
            return;
    }

    //if we are not in head, we deleted everything ahead of us
    if(currentElement > 0) {
        _history.remove(0, currentElement);
    }


    if(_history.contains(word)) {
        _history.remove(_history.indexOf(word));
    }

    //add new word to head
    _history.push_front(word);

    //fit to max size
    while(_history.size() > _maxSize) {
        _history.pop_back();
    }

    currentElement = 0;

    if(_history.size() > 1) {
        _prevAvailable = true;
        _nextAvailable = false;
        _listAvailable = true;
    }
    else {
        _prevAvailable = false;
        _nextAvailable = false;
        _listAvailable = true;
    }

    Q_EMIT historyChanged(_prevAvailable,
                        _nextAvailable,
                        _listAvailable);
}

QString History::previous() {
    if(_prevAvailable) {
        currentElement++;

        _nextAvailable = true;

        if(currentElement+1 == _history.size()) {
            _prevAvailable = false;
        }

        Q_EMIT historyChanged(_prevAvailable,
                            _nextAvailable,
                            _listAvailable);

        return _history[currentElement];
    }
    return QString();
}

QString History::next() {
    if(_nextAvailable) {
        currentElement--;

        _prevAvailable = true;

        if(currentElement == 0) {
           _nextAvailable = false;
        }

        Q_EMIT historyChanged(_prevAvailable,
                            _nextAvailable,
                            _listAvailable);

        return _history[currentElement];
    }
    return QString();
}

QStringList History::list() {
    QStringList result;

    if(_listAvailable) {
        for(int i=0; i<_history.size(); i++) {
            result << _history[i];
        }
    }
    return result;
}

bool History::nextAvailable() {
    return _nextAvailable;
}

bool History::prevAvailable() {
    return _prevAvailable;
}

bool History::listAvailable() {
    return _listAvailable;
}

void History::setCurrentElement(int element) {
    if(element < 0 || element >= _history.size()) return;

    currentElement = element;

    if(currentElement > 0) {
       _nextAvailable = true;
    }
    else {
        _nextAvailable = false;
    }

    if(currentElement+1 < _history.size()) {
        _prevAvailable = true;
    }
    else {
        _prevAvailable = false;
    }

    Q_EMIT historyChanged(_prevAvailable,
                        _nextAvailable,
                        _listAvailable);
}

void History::refreshStatus() {
    Q_EMIT historyChanged(_prevAvailable,
                        _nextAvailable,
                        _listAvailable);
}
