/*******************************************************************************

    This file is part of mDictionary.

    mDictionary is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    mDictionary is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with mDictionary.  If not, see <http://www.gnu.org/licenses/>.

    Copyright 2010 Comarch S.A.

*******************************************************************************/

/*! \file backbone.cpp
\brief Backbone/core main header \see Backbone


\author Bartosz Szatkowski <bulislaw@linux.com>
*/

#ifndef BACKBONE_H
#define BACKBONE_H

#include <QObject>
#include <QList>
#include <QHash>
#include <QSet>
#include <QPluginLoader>
#include <QFuture>
#include <QtConcurrentRun>
#include <QtConcurrentMap>
#include <QFutureIterator>
#include <QTimer>
#include <QTime>
#include <QDir>
#include <QThread>
#include <QDebug>
#include <QSettings>
#include <QFutureWatcher>
#include <QMultiMap>
#include "../../include/CommonDictInterface.h"
#include "../../include/settings.h"
#include "../../include/translation.h"
#include "../../include/History.h"
#include "../../include/Notify.h"
#include "ConfigGenerator.h"
#include "Bookmarks.h"


/*! Inner part of dictionary - glues together GUI and plugins, also kind of
    GoF facade (for GUI) cover few classes \see Bookmarks \see History

  Backbone is responsible for managing plugins and dictionaries, starting
  new searches and threads, merging search results from multiple dictionaries.

  Each plugin may live in multiple instances - each with its own dictionary,
  backbone must provide a way to create them at start (with specific Settings) and
  distinguish each ditionary.

  Backbone also manages bookmarks and history: providing interface to GUI.

  Backbone is also responsible for saving and spawning session via configs
  file (stored in ~/.mdictionary) -> configs are kind of tricky because
  mDictionary is delivered with two simple dicts -> it's necessary to separate default
  configs from user configs (updating/reinstalling app results in overwritten
  default config file), moreover config file there is general mdictionary
  configuration (apart from dictionaries and plugin ones).

  Other modules may set some internal backbone behaviour via \see setSettings():
  Settings object with options given:
     * history_size - int, size of stored searches
     * search_limit - int, how many different words each dictionary may return
     * search_dictionaries - true/false, whether search in dictionaries
     * search_bookmarks - true/false, whether search in bookmarks

    Searching schema:
        At first GUI should ask for a list of words matching a given pattern,
        then each Translation object is capable of finding its own final translation

      List of words:
        - GUI calls search(...)
        - Backbone calls plugins searchWordList(...) in idealThreadCount()+1 threads
        - Backbone sets the FutureWatcher to be notified when plugins are done
        - Backbone fetches results from Future<..> and formats it for GUI, then
           emits ready()
        - GUI calls result()

      Final translation:
         - GUI calls searchXml()
         - Backbone starts toXml for each translation object in separate threads
         - Backbone sets FutureWatcher to be notified after last toXml returns
         - Backbone fetches translation from Future<...> objects and calls
             xmlReady()
         - Gui calls xmlResult()

*/
class Backbone : public QObject
{
    Q_OBJECT

public:
    /*! \param pluginPath path to plugins (leave blank for default)
        \param configPath path to folder with configuration files
        \param dry dry run is mode without paying attention to configuration etc
          mainly for testing
      */
    Backbone(QString pluginPath="", QString configPath="",
             bool dry = 0, QObject *parent = 0);
    ~Backbone();
    Backbone(const Backbone& b);

    //! \return all loaded dictionaries with activity state flag
    QHash<CommonDictInterface*, bool> getDictionaries();

    //! \return all loaded plugins
    QList<CommonDictInterface*> getPlugins();

    //! \return history of performed searches
    History* history();

    //! \return search fesult
    QMultiHash<QString, Translation*> result();

    //! \return maximum number of words that plugin could find
    int searchLimit() const;

    //! \return final translation (after searching for xml)
    QStringList xmls();

    /*! maximum number of translations that each plugin may return; it must be
        public static because of QtConcurent::mapped restrictions about
        what kind of function may be used there see Qt docs */
    static int _searchLimit;



public Q_SLOTS:
    //! stops all current searches and emits searchCanceled signal
    void stopSearching();

    /*! searches for a word translation
       \param word word to be translated
      */
    void search(QString word);

    /*! sets active dictionaries (searches are performed only in active dicts)
       \param List of dictionaries to be activated
      */
    void selectedDictionaries(QList<CommonDictInterface*>);

    /*! adds a new dictionary and activates it
      \param dict dictionary to be added
      \param active decides whether searches are perfomed in given dictionaries
      */
    void addDictionary(CommonDictInterface* dict, bool active = 1);


    //! stops all current activity - emitting signal \see closeOk
    void quit();


    /*! Fired by FutureWatcher when a list of words is ready (after calling search),
        fetch Future<...> to final result
      */
    void translationReady();

    /*! Fired by FutureWatcher when search result is ready, fetch Future to
        final result
      */
    void xmlTranslationReady();

    /*! Removes a given dictionary
        \param dict dictionary to be deleted
      */
    void removeDictionary(CommonDictInterface* dict);

    /*! Saves plugins new state/configuration after each change */
    void dictUpdated();

    /*! Performs search for final translation (xml) form
      \param list of Translation* to be searched for
      */
    void searchXml(QList<Translation*>);


    /*! adds bookmarks to given translations (translation object is fetched and
      added to bookmarks data base (key and translation stored in db))
      \param translations list of Translation objects to be stored in db
      */
    void addBookmark(QList<Translation*> translations);


    /*! Removes bookmarks to given translations
      \param translations remove bookmark to these translations
      */
    void removeBookmark(QList<Translation*> translations) {
        foreach(Translation* translation, translations)
            _bookmarks.remove(translation);
    }



    /*! Removes all bookmarks
      */
    void removeAllBookmarks(){
        _bookmarks.clear();
    }


   /*! Searching a list of bookmarks may take some time, so it's moved to a
       new thread (to avoid GUI blocking), further it's consistent with ordinary
       searching a list of words (\see search)
       */
   void fetchBookmarks() {
        _result.clear();

        stopped = false;
        dictFin = 1;
        bookmarkFin = 0;

        _innerBookmarks = QtConcurrent::run(_bookmarks,
                &Bookmarks::list);
        _bookmarkSearchWatcher.setFuture(_innerBookmarks);
   }

   /*! Sets settings for backbone: history_size, search_limit,
       searching backends (search_bookmarks, search_dictionaries)
       \param settings settings object with options set
       */
    void setSettings(Settings* settings);

    /*! \return corresponding settings object with history_size, search_limit,
       searching backends (search_bookmarks, search_dictionaries)
       */
    Settings* settings();






Q_SIGNALS:
    /*! emitted when backbone is ready to close - after getting stop signal it
        should kill all threads and so on */
    void closeOk();

    //! emitted when there are search results ready to fetch
    void ready();

    //! emitted when xml result is ready to fetch
    void xmlReady();

    //! thrown when searches are stopped
    void searchCanceled();

    //! emitted when bookmark list is ready to fetch
    void bookmarksReady();

    /*! emitted by direct connection to plugins notifying signals
        \param Notify::NotifyType GUI may decide to show different types in
            different ways
        \param text of the notification
    */
    void notify(Notify::NotifyType, QString);

    void bookmarkReady();

private Q_SLOTS:
    void bookmarksListReady();


private:
    QHash<CommonDictInterface*, bool> _dicts; // List of dictionaries
    QList<CommonDictInterface*> _plugins;  // List of plugins


    QFuture<QList<Translation*> > _innerResult; //Res of concurrent word search
    QFuture<QString> _innerXmlResult;  // Result of xml search
    QFuture<QList<Translation*> > _innerBookmarks; //Res of search in bookmarks
    QFuture<QList<Translation*> > _innerListBookmarks; //Res of search in bookmarks
    QFuture<QStringList> _innerXmlBookmarks; //Xml result of bookmarks search

    QMultiHash<QString, Translation*> _result; //Final result of word search
    QStringList _xmlResult; // Final result of xml search
    QList<Translation*> _bookmarksResult; // Final result of search in bookmarks


    // Keeps track of concurent computations
    QFutureWatcher<QList<Translation*> > _resultWatcher;
    QFutureWatcher<QList<Translation*> > _bookmarkWatcher;
    QFutureWatcher<QList<Translation*> > _bookmarkSearchWatcher;
    QFutureWatcher<QString> _xmlResultWatcher;


    QString _pluginPath;
    QString _configPath;
    QString _dir;
    int _historyLen;
    qreal _zoom;

    bool dryRun; // mainly for testing - when true then doesn't bother configs etc
    bool stopped; // true when user stops searching/fetching
    bool bookmarkFin, dictFin; // informs whether given search type is ready
    bool _searchDicts, _searchBookmarks; // whether search performed in given source

    Bookmarks _bookmarks;


    void init();

    QStringList getFilesFromDir(QString dir, QStringList nameFilter);
    void loadPlugins(); //< locate and load plugins
    void loadPrefs(QString fileName);
    void loadDicts(QString fileName);

    void saveState(QSettings*, Settings*, bool, uint);
    void addInternalDictionary(CommonDictInterface*, bool);
    void savePrefs(QSettings*);

    CommonDictInterface* plugin(QString type); // searches for plugin of a given type
    QList<CommonDictInterface*> activeDicts();
    bool containsDict(uint hash) const;
    int _dictNum;

    History* _history;

    friend class BackboneTest;

};

#endif // BACKBONE_H
