#include "WordListModel.h"

WordListModel::WordListModel(QObject *parent) :
    QAbstractListModel(parent)
{
    QHash<int, QByteArray> roles;
    roles[WordRole] = "word";
    roles[IsBookmarkedRole] = "isBookmarked";
    roles[NumberRole] = "number";
    setRoleNames(roles);


}

int WordListModel::rowCount(const QModelIndex &parent) const
{
    return _wordList.count();
}

void WordListModel::addWord(QString word, QList<Translation *> translations, bool isBookmarked)
{
    beginInsertRows(QModelIndex(), rowCount(), rowCount());
    _translations.insert(word, translations);
    _wordList << word;
    _wordInBookmarks.insert(word, isBookmarked);
    endInsertRows();
}

void WordListModel::clear()
{
    bool empty = true;
    if (!_wordList.isEmpty()){
        beginResetModel();
        empty = false;
    }
    _translations.clear();
    _wordInBookmarks.clear();
    _wordList.clear();
    if (!empty){
        endResetModel();
    }
}

QVariant WordListModel::data(const QModelIndex &index, int role) const
{
    if (index.row() < 0 || index.row() > _translations.count()){
        return QVariant();
    }

    if (role == WordRole)
    {
        return _wordList[index.row()];
    }
    if (role == NumberRole){
        return index.row();
    }
    if (role == IsBookmarkedRole){
        return _wordInBookmarks[_wordList[index.row()]];
    }
    return QVariant();
}

bool WordListModel::setData(const QModelIndex &index, const QVariant &value, int role)
{
    int res = setDataPriv(index.row(), value, role);
    if (res == 0)
        return false;
    if (res > 0)
        return true;
    return true;
}

Qt::ItemFlags WordListModel::flags(const QModelIndex &index) const
{
    Qt::ItemFlags fl = QAbstractItemModel::flags(index);
    return (fl | Qt::ItemIsEditable);
}

void WordListModel::setTranslations(QHash<QString, QList<Translation *> > translations, QHash<QString, bool> wordsInBookmarks)
{
    QHashIterator<QString, QList<Translation *> > i(translations);
    while (i.hasNext()) {
        i.next();
        addWord(i.key(), i.value(), wordsInBookmarks[i.key()]);
    }

    sort(0);

}

void WordListModel::sort(int column, Qt::SortOrder order)
{
    if (column != 0)
        return;

    int left = 0;
    int right = _wordList.count() - 1;

    if (left < right){
        if (order == Qt::AscendingOrder){
            qSort(_wordList.begin(), _wordList.end());
        } else if (order == Qt::DescendingOrder) {
            qSort(_wordList.begin(), _wordList.end(), qGreater<QString>());
        }
    }
}

void WordListModel::setModelProperty(int index, const QVariant value, QString role)
{
    if (role.contains("isBookmarked"))
    {
        setDataPriv(index, value, IsBookmarkedRole);
    }
}

int WordListModel::setDataPriv(int index, const QVariant &value, int role)
{
    bool bookmarksOnly = false;
    if (_wordInBookmarks.values().count(false) == 0){
        bookmarksOnly = true;
    }

    if (index < 0 || index > _translations.count())
        return 0;

    QString word = _wordList[index];
    if (role == WordRole)
        return 1;
    if (role == NumberRole)
        return 1;
    if (role == IsBookmarkedRole)
    {
        if (value.type() == QVariant::Bool)
        {
            _wordInBookmarks[word] = value.toBool();
            Q_EMIT dataChanged(this->index(0), this->index(_translations.count() - 1));
            if (_wordInBookmarks[word] == true){
                Q_EMIT addToBookmarks(word);
            } else {                
                Q_EMIT removeFromBookmarks(word);
                if (bookmarksOnly == true){
                    beginRemoveRows(QModelIndex(), index, index + 1);
                    this->_translations.remove(_wordList[index]);
                    this->_wordInBookmarks.remove(_wordList[index]);
                    this->_wordList.removeAt(index);
                    endRemoveRows();
                }
            }
            return 2;
        }
        else
        {
            return 0;
        }
    }
    return 0;
}
