/*******************************************************************************

    This file is part of mDictionary.

    mDictionary is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    mDictionary is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with mDictionary.  If not, see <http://www.gnu.org/licenses/>.

    Copyright 2010 Comarch S.A.

*******************************************************************************/
/*!
    \file GoogleDialog.h
    \brief Implementation of google plugin's dialogs.

    \author Mateusz Półrola <mateusz.polrola@gmail.com>
    \author Jakub Jaszczynski <j.j.jaszczynski@gmail.com>
*/

#ifndef GOOGLEDIALOG_H
#define GOOGLEDIALOG_H

#include <QDialog>
#include "../../include/settings.h"
#include "../../include/ComboBoxModel.h"
#include <QtGui>


#include <QDeclarativeView>
#include <QDeclarativeContext>
#include <QList>

#include "GooglePlugin.h"

/*!
    This class can create dialogs for adding a new dictionary or changing settings
     of an existing one, based on dialog type passed to constructor.
     These types differ only in confirm button label.
     Both provide comboboxes with available languages to choose.
*/
class GoogleDialog : public QDialog
{
    Q_OBJECT
public:
    /*!
      Describes type of dialog. New means that dialog confirm button has
        "Add" label, Change means that dialog confirm button has "Save settings" label.
    */
    enum GoogleDialogType {New, Change};

    //! Constructor
    /*!
        Creates new google dialog
        \param plugin if created dialog is of type Change it must be set to
            point to plugin whose settings will be changed
        \param type describes type of created dialog
        \param parent parent widget of created dialog
    */
    explicit GoogleDialog(GooglePlugin* plugin = 0,
                          GoogleDialogType type = New,
                          QWidget* parent = 0);

    //! \returns settings of plugin
    /*!
        After acceptance of dialog this method returns plugin's settings based on
         user's choices in dialog.
    */
    Settings* getSettings();

Q_SIGNALS:
    //! requests to show notification
    void notify(Notify::NotifyType, QString);

#ifndef Q_WS_MAEMO_5
    void setInfo(QVariant info);
    void setNew(QVariant text);
    void setStartValues(QVariant from, QVariant to, QVariant fromIndex, QVariant toIndex);
#endif

public Q_SLOTS:
#ifdef Q_Ws_MAEMO_5
    //! reimplemented accept method, to save new settings
    void accept();
#endif

private Q_SLOTS:
    //! assigns the language chosen from a list(langFromComboBox) to _langFrom
    void langFromChanged(int);

    //! assigns the language chosen from a list(langToComboBox) to _langTo
    void langToChanged(int);

    //! handles the "swap languages" button
    void changeLangButtonClicked();


#ifndef Q_Ws_MAEMO_5
    void saveButtonClicked(QString langFrom, QString langTo);
#endif

private:

    void initializeUI();

    //! saves new settings after acceptance of dialog
#ifdef Q_Ws_MAEMO_5
    void saveSettings();
#else
    void saveSettings(QString langFrom, QString langTo);

    ComboBoxModel* model;
    QVBoxLayout* mainLayout;
    QDeclarativeView *view;
    QDeclarativeContext* ctxt;
#endif

    int lastHeight;

    QLabel* infoLabel;
    QLabel* langFromLabel;
    QLabel* langToLabel;
    QLabel* connectInfoLabel;
    QPushButton* confirmButton;
    QPushButton* changeLangButton;
    QComboBox *langFromComboBox;
    QComboBox *langToComboBox;
    QVBoxLayout* verticalLayout;
    QVBoxLayout* langLayout;
    QFormLayout* langsFormLayout;
    QHBoxLayout* changeLangLayout;
    QString _langFrom;
    QString _langTo;
    int _actualLangTo;
    int _actualLangFrom;

    Settings* _settings;
    GooglePlugin* plugin;
    GoogleDialogType type;
};

#endif // GOOGLEDIALOG_H
