/*******************************************************************************

    This file is part of mDictionary.

    mDictionary is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    mDictionary is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with mDictionary.  If not, see <http://www.gnu.org/licenses/>.

    Copyright 2010 Comarch S.A.

*******************************************************************************/
/*!
    \file StarDialog.h
    \brief Implementation of stardict plugin's dialogs.

    \author Mateusz Półrola <mateusz.polrola@gmail.com>
*/

#ifndef STARDIALOG_H
#define STARDIALOG_H

#include <QDialog>
#include "../../include/settings.h"
#include <QtGui>
#include <QDeclarativeView>
#include "StarDictPlugin.h"


/*!
    This class can create dialogs for adding a new dictionary or changing settings
    of an existing one, based on dialog type passed to contructor.
    When adding a new dictionary dialog contains a button to browse file system and
    select a dictionary file. When changing settings dialog displays basic
    information about dictionary i. e. name, languages and license info.
    All contents of a dialog are in a scroll area.
*/
class StarDialog : public QDialog
{
    Q_OBJECT
public:
    /*!
        Describes type of dialog. New means that dialog contains widgets to browse
        file system and select dictionary file. Change means that dialog displays
        information about dictionary.
        In both types dialog provides widgets to create or delete cache and remove
        or keep accents.
    */
    enum StarDialogType {New, Change};

    //! Constructor
    /*!
        Creates new StarDict dialog
        \param plugin if created dialog is of type Change it must be set to
            point to plugin whose settings will be changed
        \param type describes type of created dialog
        \param parent parent widget of created dialog
    */
    explicit StarDialog(StarDictPlugin* plugin = 0,
                        StarDialogType type = New,
                        QWidget* parent = 0);

    /*!
        After acceptance of dialog this method returns plugin's settings based on
        user's choices in dialog.
        \returns settings of plugin
    */
    Settings* getSettings();

Q_SIGNALS:
    //! Requests to show notification
    void notify(Notify::NotifyType, QString);

    void setInfo(QVariant info);
    void setButtonText(QVariant text);
    void setNew(QVariant text);
    void setPath(QVariant path);

public Q_SLOTS:
    /*!
        Reimplemented accept method, to check if all necessary fields in
        dialog are correct e. g. dictionary file path
        and saves new settings
    */
    void accept();



private Q_SLOTS:
    void resizeEvent(QResizeEvent *e);

    //! displays dialog to browse and select file
    void selectFile();

    void heightChange(int height);

private:
    QVBoxLayout* mainLayout;
    QDeclarativeView *view;
    int lastHeight;

    //! create a user interface
    void initializeUI();

    //! saves new settings after acceptance of dialog
    void saveSettings();

    //! check that files is compressed
    bool checkFiles();

    QLabel* infoLabel;
    QPushButton* browseButton;
    QHBoxLayout* browseLayout;

    QPushButton* confirmButton;
    QString _dictionaryFilePath;
    QScrollArea* scrollArea;
    QWidget* widget;
    QHBoxLayout* layout;
    QVBoxLayout* mainVerticalLayout;
    bool _isCompressed;
    QString _dictName;

    Settings* _settings;

    StarDictPlugin* plugin;
    StarDialogType type;
};

#endif // STARDIALOG_H
