/*******************************************************************************

    This file is part of mDictionary.

    mDictionary is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    mDictionary is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with mDictionary.  If not, see <http://www.gnu.org/licenses/>.

    Copyright 2010 Comarch S.A.

*******************************************************************************/
/*!
    \file XdxfDictDownloader.cpp
    \brief XdxfDictDownloader is responsible for getting dict list from XDXF website
    and other actions necessary to download and add dictionary

    \author Mateusz Półrola <mateusz.polrola@comarch.com>
*/

#include "XdxfDictDownloader.h"
#include "XdxfDictDownloadProgressDialog.h"
#include <QDebug>
#include <QProcess>
#include <bzlib.h>
#include <stdio.h>
#include <fcntl.h>

#ifndef Q_WS_MAEMO_5
    #include <libtar.h>
#endif


typedef void BZFILE;


XdxfDictDownloader::XdxfDictDownloader(QObject *parent) :
    QObject(parent) {
    parentDialog = 0;
    progressDialog = 0;
    manager = new QNetworkAccessManager(this);

    connect(manager, SIGNAL(finished(QNetworkReply*)),
            this, SLOT(dictListReceived(QNetworkReply*)));
    connect(&http, SIGNAL(finished()),
            this, SLOT(processFinished()));
    connect(&http, SIGNAL(error(QString)),
            this, SLOT(downloadingError(QString)));
    connect(&http, SIGNAL(progress(qint64,qint64)),
            this, SLOT(updateDownloadProgress(qint64,qint64)));
}


void XdxfDictDownloader::download(QWidget *parent) {
    parentDialog = parent;
    aborted = false;

    manager->get(QNetworkRequest(QUrl("http://xdxf.revdanica.com/down/")));

    progressDialog = new XdxfDictDownloadProgressDialog(parent);

    connect(progressDialog, SIGNAL(cancelDownloading()),
            this, SLOT(breakDownloading()));
    connect(this, SIGNAL(downloadProgress(float)),
            progressDialog, SLOT(updateProgress(float)));

    qDebug()<<"etam 3.1";
    progressDialog->setText(tr("Downloading dictionaries list"));
    progressDialog->show();
}


QString XdxfDictDownloader::downloadedFile() {
    return _downloadedFile;
}


void XdxfDictDownloader::updateDownloadProgress(qint64 downloaded,
                                                qint64 total)   {
    Q_EMIT downloadProgress(float(downloaded) / float(total));
}


void XdxfDictDownloader::downloadingError(QString error) {
    breakDownloading();
    Q_EMIT notify(Notify::Error, error);
}


void XdxfDictDownloader::breakDownloading() {
    //if user cancel downloading we kill all running processes, hide progress dialog and set flag that user cancel downloading.
    aborted = true;
    http.kill();

    if(progressDialog && progressDialog->isVisible()) {
        progressDialog->accept();
    }
}


void XdxfDictDownloader::processFinished() {
    //first check if user cancel downloading
    if(aborted) return;

    if(!extract("/tmp/" + _fileName)) {
        Q_EMIT notify(Notify::Error,
                "Error while extracting dictionary archive");
        return;
    }
    downloadComplete();
}


void XdxfDictDownloader::downloadComplete() {
    if(aborted) return;
    // Downloaded tar file name is different than extracted folder so we need
    // some clean directory to identify extracted files
    QDir dir("/tmp/mdict");
    QString dictDirName = dir.entryList().at(2);

    // Dict is in /tmp/mdict/<extracted directory>/dict.xdxf
    QFile dictFile("/tmp/mdict/" + dictDirName + "/dict.xdxf");
    dictFile.copy(QDir::homePath() + "/.mdictionary/" + dictDirName + ".xdxf");
    QFile::remove("/tmp/" + _fileName);
    QFile::remove("/tmp/" + _fileName.replace(QRegExp(".bz2$"), ""));

    _downloadedFile = QDir::homePath() + "/.mdictionary/" + dictDirName + ".xdxf";

    progressDialog->accept();
    delete progressDialog;
    progressDialog = 0;

    emit fileDownloaded(_downloadedFile);
}


void XdxfDictDownloader::dictListReceived(QNetworkReply *reply) {
    progressDialog->accept();
    if(aborted)
        return;
    if(reply->error() != QNetworkReply::NoError) {
        Q_EMIT notify(Notify::Error, reply->errorString());
        return;
    }

    QString page(QString::fromUtf8(reply->readAll()));

    // You can look at http://xdxf.revdanica.com/down/, we need to get table
    // with dictionaries entries following regexp match its begining
    QRegExp regOuter("<td>Icon</td><td>Name</td><td>Archive filename</td><td>Archive file size</td><td>Dict file size</td><td>Number of articles</td><td>From</td><td>To</td><td>Submitted by</td><td>Submition date</td></tr>(.*)</table>");
    regOuter.setMinimal(true);
    if(!regOuter.indexIn(page))
        return;

    // Cutting each entry and creating coresponded DownloadDict object
    page = regOuter.capturedTexts().at(1);
    QRegExp regInner("<tr>.*</tr>");
    regInner.setMinimal(true);
    int pos = 0;
    dicts.clear();
    while ((pos = regInner.indexIn(page, pos)) != -1) {
        DownloadDict temp = DownloadDict(regInner.cap(0));
        if(!temp.fromLang().isEmpty())
            dicts.append(temp);
        pos += regInner.matchedLength();
    }

    XdxfDictSelectDialog selectDialog(dicts, parentDialog);

    if(selectDialog.exec()==QDialog::Accepted) {
        qDebug()<<"etap 3.2";
        progressDialog->setText(tr("Downloading dictionary"));
        progressDialog->show();

        QString url = selectDialog.link();
        _fileName = url.split('/').last();

        QProcess clean;
        clean.start("rm -rf /tmp/mdict");
        clean.waitForFinished(-1);
        clean.start("mkdir /tmp/mdict");
        clean.waitForFinished(-1);

        http.download(QUrl(url), "/tmp/" + _fileName);
    }
}

bool XdxfDictDownloader::extract(QString file) {
    // Extracting bz2
    FILE * archive = fopen(file.toStdString().c_str(), "rb");
    if (archive == 0)
        return false;
    int err;
    BZFILE * afterbzFile = BZ2_bzReadOpen(&err, archive, 0, 0, 0, 0);
    if(err != BZ_OK) {
        BZ2_bzReadClose(&err, afterbzFile);
        return false;
    }

    FILE * tarfile = fopen(file.replace(QRegExp(".bz2$"), "").
            toStdString().c_str(), "w");

    int bufflen = 100;
    char buff[bufflen];
    while(err == BZ_OK) {
        unsigned int len = BZ2_bzRead(&err, afterbzFile, buff, bufflen);
        if(fwrite(buff, 1, len, tarfile) != len)
            return false;
    }
    BZ2_bzReadClose(&err, afterbzFile);
    fclose(tarfile);
    fclose(archive);

    // Extracting tar
    #ifndef Q_WS_MAEMO_5
    TAR *t;
    char * tarfname = new char[file.replace(QRegExp(".bz2%"), "").size()+1];
    strcpy(tarfname, file.replace(QRegExp(".bz2%"), "").toStdString().c_str());

    err = tar_open(&t, tarfname, 0, O_RDONLY, 0, 0);
    if(err == -1)
        return false;

    char text[]={"/tmp/mdict/"};
    err = tar_extract_all(t,text);
    if(err == -1) {
        return false;
    }
    tar_close(t);
    #else
    QProcess tar;
    tar.start("tar -xvf " + file.replace(QRegExp(".bz2%"), "") + " -C /tmp/mdict");
    tar.waitForFinished(-1);
    #endif

    return true;
}



