/*******************************************************************************

    This file is part of mDictionary.

    mDictionary is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    mDictionary is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with mDictionary.  If not, see <http://www.gnu.org/licenses/>.

    Copyright 2010 Comarch S.A.

*******************************************************************************/
/*!
    \file xdxfplugin.cpp
    \brief Implementation of xdxf plugin's main class.

    \author Jakub Jaszczynski <j.j.jaszczynski@gmail.com>
*/

#include "xdxfplugin.h"
#include <QDebug>
#include "../../include/Notify.h"
#include "DownloadDict.h"
#include "XdxfDictDownloader.h"

XdxfDictDownloader XdxfPlugin::dictDownloader;


bool XdxfPlugin::dictDownloaderInitialized = false;


XdxfPlugin::XdxfPlugin(QObject *parent) : CommonDictInterface(parent),
                    _langFrom(""), _langTo(""),_name(""), _infoNote("") {
    _settings = new Settings();
    _dictDialog = new XdxfDictDialog(this, this);

    connect(_dictDialog, SIGNAL(notify(Notify::NotifyType,QString)),
            this, SIGNAL(notify(Notify::NotifyType,QString)));

    if(!dictDownloaderInitialized) {
        connect(&dictDownloader, SIGNAL(notify(Notify::NotifyType,QString)),
                this, SIGNAL(notify(Notify::NotifyType,QString)));
        dictDownloaderInitialized = true;
    }

    _settings->setValue("type","xdxf");
    _iconPath = "/usr/share/mdictionary/xdxf.png";
    _icon = QIcon(_iconPath);
    _wordsCount = -1;
    stopped = false;

    initAccents();
}


void XdxfPlugin::retranslate() {
    QString locale = QLocale::system().name();
    QTranslator *translator = new QTranslator(this);

    if(!translator->load(":/xdxf/translations/" + locale)) {
        translator->load(":/xdxf/translations/en_US");
    }
    QCoreApplication::installTranslator(translator);
}


XdxfPlugin::~XdxfPlugin() {
    delete _settings;
    delete _dictDialog;
}


QString XdxfPlugin::langFrom() const {   
    return _langFrom;
}


QString XdxfPlugin::langTo() const {
    return  _langTo;
}


QString XdxfPlugin::name() const {
    return  _name;
}


QString XdxfPlugin::type() const {
    return QString("xdxf");
}


QString XdxfPlugin::infoNote() const {
    return _infoNote;
}


QList<Translation*> XdxfPlugin::searchWordList(QString word, int limit) {
    if( word.indexOf("*")==-1 && word.indexOf("?")==-1 &&
        word.indexOf("_")==-1 && word.indexOf("%")==-1)
        word+="*";

    if(isCached())
        return searchWordListCache(word,limit);
    return searchWordListFile(word, limit);
}


QList<Translation*> XdxfPlugin::searchWordListCache(QString word, int limit) {
    QSet<Translation*> translations;
    QString cacheFilePath = _settings->value("cache_path");

    db.setDatabaseName(cacheFilePath);
    if(!QFile::exists(cacheFilePath) || !db.open()) {
        qDebug() << "Database error" << db.lastError().text() << endl;
        Q_EMIT notify(Notify::Warning, QString(tr("Cache database cannot be "
                "opened for %1 dictionary. Searching in XDXF file. "
                "You may want to recache.").arg(name())));
        _settings->setValue("cached","false");
        return searchWordListFile(word, limit);
    }

    stopped = false;
    word = word.toLower();
    word = word.replace("*", "%");
    word = word.replace("?", "_");

    QSqlQuery cur(db);
    if(limit !=0)
        cur.prepare("select word from dict where word like ? or normalized "
                    "like ? limit ?");
    else
        cur.prepare("select word from dict where word like ? or normalized "
                    "like ?");
    cur.addBindValue(word);
    cur.addBindValue(word);
    if(limit !=0)
        cur.addBindValue(limit);
    cur.exec();

    while(cur.next() && (translations.size()<limit || limit==0)) {
       translations.insert(new TranslationXdxf(
            cur.value(0).toString(),
            _dictionaryInfo, this));
    }
    db.close();
    return translations.toList();
}


QList<Translation*> XdxfPlugin::searchWordListFile(QString word, int limit) {
    QSet<Translation*> translations;
    QFile dictionaryFile(_settings->value("path"));
    word = word.toLower();
    stopped = false;

    QRegExp regWord(word);
    regWord.setCaseSensitivity(Qt::CaseInsensitive);
    regWord.setPatternSyntax(QRegExp::Wildcard);

    /*check xdxf file exist*/
    if(!QFile::exists(_settings->value("path"))
                || !dictionaryFile.open(QFile::ReadOnly | QFile::Text)) {
        qDebug()<<"Error: could not open file";
        Q_EMIT notify(Notify::Warning,
                QString(tr("XDXF file cannot be read for %1").arg(name())));
        return translations.toList();
    }

    QXmlStreamReader reader(&dictionaryFile);
    QString readKey;
    int i=0;

    /*search words list*/
    while(!reader.atEnd() && !stopped){
        reader.readNextStartElement();
        if(reader.name()=="ar") {
            while(reader.name()!="k" && !reader.atEnd())
                reader.readNextStartElement();
            if(!reader.atEnd())
                readKey = reader.readElementText();
            if((regWord.exactMatch(readKey)
                    || regWord.exactMatch(removeAccents(readKey)))
                    && (i<limit || limit==0) && !reader.atEnd())  {
                translations<<(new TranslationXdxf(readKey.toLower(),
                               _dictionaryInfo,this));
                if(translations.size()==limit && limit!=0)
                    break;
            }
        }
        this->thread()->yieldCurrentThread();
    }
    stopped=false;
    dictionaryFile.close();
    return translations.toList();
}


QString XdxfPlugin::search(QString key) {
    if(isCached())
        return searchCache(key);
    return searchFile(key);
}


QString XdxfPlugin::searchCache(QString key) {
    QString result("");
    QString cacheFilePath = _settings->value("cache_path");
    db.setDatabaseName(cacheFilePath);
    key = key.toLower();

    if(!QFile::exists(cacheFilePath) || !db.open()) {
        qDebug() << "Database error" << db.lastError().text() << endl;
        Q_EMIT notify(Notify::Warning, QString(tr("Cache database cannot be "
                "opened for %1 dictionary. Searching in XDXF file. "
                "You may want to recache.").arg(name())));
        _settings->setValue("cached","false");
        return searchFile(key);
    }

    QSqlQuery cur(db);

    cur.prepare("select translation from dict where word like ?");
    cur.addBindValue(key);
    cur.exec();
    while(cur.next())
        result += cur.value(0).toString();
    db.close();

    return result;
}


QString XdxfPlugin::searchFile(QString key) {
    QFile dictionaryFile(_settings->value("path"));
    QString resultString("");
    key = key.toLower();

    /*check xdxf file exist*/
    if(!QFile::exists(_settings->value("path"))
                || !dictionaryFile.open(QFile::ReadOnly | QFile::Text)) {
        Q_EMIT notify(Notify::Warning,
                QString(tr("XDXF file cannot be read for %1").arg(name())));
        qDebug()<<"Error: could not open file";
        return "";
    }

    QXmlStreamReader reader(&dictionaryFile);
    QString readKey;
    bool match =false;
    stopped = false;

    /*search translations for word*/
    while (!reader.atEnd()&& !stopped) {
        reader.readNext();
        if(reader.tokenType() == QXmlStreamReader::StartElement) {
            if(reader.name()=="k") {
                readKey = reader.readElementText();
                if(readKey.toLower()==key.toLower())
                    match = true;
            }
        }
        if(match) {
            QString temp("");
            while(reader.name()!="ar" && !reader.atEnd()) {
                if(reader.name()!="" && reader.name()!="k") {
                    if(reader.tokenType()==QXmlStreamReader::EndElement)
                        temp+="</";
                    if(reader.tokenType()==QXmlStreamReader::StartElement)
                        temp+="<";
                    temp+=reader.name().toString();
                    if(reader.name().toString()=="c" &&
                            reader.tokenType()==QXmlStreamReader::StartElement)
                       temp= temp + " c=\"" + reader.attributes().
                               value("c").toString() + "\"";
                    temp+=">";
                }
                temp+= reader.text().toString().replace("<","&lt;").
                        replace(">","&gt;");
                reader.readNext();
            }
            if(temp.at(0)==QChar('\n'))
                temp.remove(0,1);
            resultString+="<key>" + readKey +"</key>";
            resultString+="<t>" + temp + "</t>";
            match=false;
        }
        this->thread()->yieldCurrentThread();
    }
    stopped=false;
    dictionaryFile.close();
    return resultString;
}


void XdxfPlugin::stop() {
    stopped=true;
}


DictDialog* XdxfPlugin::dictDialog() {
     return _dictDialog;
}


CommonDictInterface* XdxfPlugin::getNew(const Settings *settings) const {
    XdxfPlugin *plugin = new XdxfPlugin();

    connect(plugin, SIGNAL(notify(Notify::NotifyType,QString)),
            this, SIGNAL(notify(Notify::NotifyType,QString)));

    ((XdxfDictDialog*)plugin->dictDialog())->setLastDialogParent(_dictDialog->lastDialogParent());

    if(settings && plugin->setSettings(settings)) {
        disconnect(plugin, SIGNAL(notify(Notify::NotifyType,QString)),
                this, SIGNAL(notify(Notify::NotifyType,QString)));
        return plugin;
    }
    else {
        disconnect(plugin, SIGNAL(notify(Notify::NotifyType,QString)),
                this, SIGNAL(notify(Notify::NotifyType,QString)));
        delete plugin;
        return 0;
    }
}


bool XdxfPlugin::isAvailable() const {
    return true;
}


Settings* XdxfPlugin::settings() {
    return _settings;
}


bool XdxfPlugin::isCached() {
    if(_settings->value("cached") == "true")
        return true;
    return false;
}


bool XdxfPlugin::setSettings(const Settings *settings) {
    if(settings) {
        bool isPathChange=false;
        QString oldPath = _settings->value("path");
        Settings *oldSettings =  new Settings ;

        if(oldPath != settings->value("path")) {
            if(oldPath!="" && _settings->value("cache_path")!="")
                clean();
            isPathChange=true;
        }

        foreach(QString key, _settings->keys())
            oldSettings->setValue(key, _settings->value(key));

        foreach(QString key, settings->keys())
           if(key != "generateCache")
               _settings->setValue(key, settings->value(key));

        if(!getDictionaryInfo()) {
            Q_EMIT notify(Notify::Warning,
                QString(tr("XDXF file is in wrong format")));
            qDebug()<<"Error: xdxf file is in wrong format";
            delete _settings;
            _settings=oldSettings;
            return false;
        }

        if(isPathChange) {
            _wordsCount=0;
            if(oldPath!="")
                _settings->setValue("cached","false");
            if(_settings->value("cached")=="true"
                    && _settings->value("cache_path")!="") {
                db_name = _settings->value("type")
                        + _settings->value("cache_path");
                db = QSqlDatabase::addDatabase("QSQLITE",db_name);
            }
        }

        if((_settings->value("cached") == "false" ||
            _settings->value("cached").isEmpty()) &&
            settings->value("generateCache") == "true") {
            clean();
            makeCache("");
        }

        else if (settings->value("generateCache") == "false") {
            _settings->setValue("cached", "false");
        }
    }
    else
        return false;
    Q_EMIT settingsChanged();
    return true;
}


bool XdxfPlugin::getDictionaryInfo() {
    QFile dictionaryFile(_settings->value("path"));
    if(!QFile::exists(_settings->value("path"))
                || !dictionaryFile.open(QFile::ReadOnly | QFile::Text)) {
       Q_EMIT notify(Notify::Warning,
               QString(tr("XDXF dictionary cannot be read from file")));
        qDebug()<<"Error: could not open file";
        return false;
    }

    bool okFormat=false;
    QXmlStreamReader reader(&dictionaryFile);
    reader.readNextStartElement();
    if(reader.name()=="xdxf") {
        okFormat=true;
        if(reader.attributes().hasAttribute("lang_from"))
            _langFrom = reader.attributes().value("lang_from").toString();
        if(reader.attributes().hasAttribute("lang_to"))
            _langTo = reader.attributes().value("lang_to").toString();
    }
    reader.readNextStartElement();
    if(reader.name()=="full_name")
        _name=reader.readElementText();
    else
        qDebug()<<"no full_name";
    reader.readNextStartElement();
    if(reader.name()=="description")
        _infoNote=reader.readElementText();
    else
        qDebug()<<"no description";

    _dictionaryInfo= _name + " [" + _langFrom + "-"
                + _langTo + "]";

    dictionaryFile.close();
    if(okFormat)
        return true;
    return false;
}


QIcon* XdxfPlugin::icon() {
    return &_icon;
}

QString XdxfPlugin::iconPath(){
    return _iconPath;
}


int XdxfPlugin::countWords() {
    if(_wordsCount>0)
        return _wordsCount;
    QFile dictionaryFile(_settings->value("path"));
    if(!QFile::exists(_settings->value("path"))
                || !dictionaryFile.open(QFile::ReadOnly | QFile::Text)) {
        Q_EMIT notify(Notify::Warning,
                QString(tr("XDXF file cannot be read for %1 dictionary")
                .arg(name())));
        qDebug()<<"Error: could not open file";
        return -1;
    }

    dictionaryFile.seek(0);
    long wordsCount = 0;

    QString line;
    while(!dictionaryFile.atEnd()) {
        line = dictionaryFile.readLine();
        if(line.contains("<k>")) {
            wordsCount++;
        }
    }
    _wordsCount = wordsCount;
    dictionaryFile.close();
    return wordsCount;
}


bool XdxfPlugin::makeCache(QString) {
    XdxfCachingDialog d(_dictDialog->lastDialogParent());

    connect(&d, SIGNAL(cancelCaching()),
            this, SLOT(stop()));
    connect(this, SIGNAL(updateCachingProgress(int,int)),
            &d, SLOT(updateCachingProgress(int,int)));

    d.show();
    QCoreApplication::processEvents();
    QFileInfo dictFileN(_settings->value("path"));
    QString cachePathN;
    stopped = false;

    /*create cache file name*/
    int i=0;
    do {
        cachePathN = QDir::homePath() + "/.mdictionary/"
                                      + dictFileN.completeBaseName()+"."
                                      +QString::number(i) + ".cache";
        i++;
    } while(QFile::exists(cachePathN));

    db_name = _settings->value("type") + cachePathN;
    db = QSqlDatabase::addDatabase("QSQLITE",db_name);

    /*checke errors (File open and db open)*/
    QFile dictionaryFile(dictFileN.filePath());
    if (!QFile::exists(_settings->value("path"))
                || !dictionaryFile.open(QFile::ReadOnly | QFile::Text)) {
        Q_EMIT updateCachingProgress(100, 0);
        Q_EMIT notify(Notify::Warning,
                QString(tr("XDXF file cannot be read for %1 dictionary")
                .arg(name())));
        return 0;
    }
    QXmlStreamReader reader(&dictionaryFile);
    db.setDatabaseName(cachePathN);
    if(!db.open()) {
        qDebug() << "Database error" << db.lastError().text() << endl;
        Q_EMIT updateCachingProgress(100, 0);
        Q_EMIT notify(Notify::Warning, QString(tr("Cache database cannot be "
                "opened for %1 dictionary. Searching in XDXF file. "
                "You may want to recache.").arg(name())));
        return false;
    }

    /*inicial sqlQuery*/
    QCoreApplication::processEvents();
    QSqlQuery cur(db);
    cur.exec("PRAGMA synchronous = 0");
    cur.exec("drop table dict");
    QCoreApplication::processEvents();
    cur.exec("create table dict(word text, normalized text ,translation text)");
    int counter = 0;
    cur.exec("BEGIN;");

    QString readKey;
    bool match = false;
    QTime timer;
    timer.start();
    countWords();
    int lastProg = -1;
    _settings->setValue("strip_accents", "true");
    counter=0;

    /*add all words to db*/
    while (!reader.atEnd() && !stopped) {
        QCoreApplication::processEvents();
        reader.readNext();
        if(reader.tokenType() == QXmlStreamReader::StartElement) {
            if(reader.name()=="k"){
                readKey = reader.readElementText();
                match = true;
            }
        }
        if(match) {
            QString temp("");
            while(reader.name()!="ar" && !reader.atEnd()) {
                if(reader.name()!="" && reader.name()!="k") {
                    if(reader.tokenType()==QXmlStreamReader::EndElement)
                        temp+="</";
                    if(reader.tokenType()==QXmlStreamReader::StartElement)
                        temp+="<";
                    temp+=reader.name().toString();
                    if(reader.name().toString()=="c"
                        && reader.tokenType()==QXmlStreamReader::StartElement) {
                        temp= temp + " c=\""
                                   + reader.attributes().value("c").toString()
                                   + "\"";
                    }
                    temp+=">";
                }
                temp+= reader.text().toString().replace("<","&lt;").replace(">"
                              ,"&gt;");
                reader.readNext();
            }
            if(temp.at(0)==QChar('\n'))
                temp.remove(0,1);
            temp="<key>" + readKey + "</key>" + "<t>" + temp+ "</t>";
            match=false;
            cur.prepare("insert into dict values(?,?,?)");
            cur.addBindValue(readKey.toLower());
            cur.addBindValue(removeAccents(readKey).toLower());
            cur.addBindValue(temp);
            cur.exec();
            counter++;
            int prog = counter*100/_wordsCount;
            if(prog % 2 == 0 && lastProg != prog) {
                Q_EMIT updateCachingProgress(prog,timer.restart());
                lastProg = prog;
            }
        }
    }
    cur.exec("END;");
    cur.exec("select count(*) from dict");

    /*checke errors (wrong number of added words)*/
    countWords();
    if(!cur.next() || countWords() != cur.value(0).toInt()) {
        Q_EMIT updateCachingProgress(100, timer.restart());
        Q_EMIT notify(Notify::Warning,
                QString(tr("Database caching error, please try again.")));
        db.close();
        _settings->setValue("cache_path", cachePathN);
        if(stopped)
            clean();
        _settings->setValue("cache_path","");
        return false;
    }

    _settings->setValue("cache_path", cachePathN);
    _settings->setValue("cached", "true");

    disconnect(&d, SIGNAL(cancelCaching()),
            this, SLOT(stop()));
    disconnect(this, SIGNAL(updateCachingProgress(int,int)),
            &d, SLOT(updateCachingProgress(int,int)));
    db.close();
    return true;
}


void XdxfPlugin::clean() {
    if(QFile::exists(_settings->value("cache_path"))) {
        QFile(_settings->value("cache_path")).remove();
        QSqlDatabase::removeDatabase(db_name);
    }
}


Q_EXPORT_PLUGIN2(xdxf, XdxfPlugin)
