/*******************************************************************************

    This file is part of mDictionary.

    mDictionary is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    mDictionary is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with mDictionary.  If not, see <http://www.gnu.org/licenses/>.

    Copyright 2010 Comarch S.A.

*******************************************************************************/
/*!
    \file xdxfplugin.h
    \brief Implementation of xdxf plugin's main class.

    \author Jakub Jaszczynski <j.j.jaszczynski@gmail.com>
*/

#ifndef XDXFPLUGIN_H
#define XDXFPLUGIN_H

#include <QObject>
#include <QDialog>
#include <QRegExp>
#include <QTime>
#include <QSqlQuery>
#include <QSqlDatabase>
#include <QSqlError>
#include <QFile>
#include <QXmlStreamReader>
#include <QtPlugin>
#include <QHash>

#include "../../include/CommonDictInterface.h"
#include "../../include/settings.h"
#include "XdxfDictDialog.h"
#include "XdxfCachingDialog.h"
#include "TranslationXdxf.h"
#include "XdxfDictDownloader.h"

class TranslationXdxf;

class XdxfPlugin : public CommonDictInterface
{
    Q_OBJECT
    Q_INTERFACES(CommonDictInterface)
public:
    XdxfPlugin(QObject *parent=0);
    ~XdxfPlugin();

    //! \returns source language code iso 639-2
    QString langFrom() const;

    //! \returns destination language code iso 639-2
    QString langTo() const;

    //! \returns dictionary name (like "old English" or so)
    QString name() const;

    //! \returns dictionary type (xdxf, google translate, etc)
    QString type() const;

    //! \returns information about dictionary in xml (name, authors, etc)
    QString infoNote() const;

    /*!
        \returns DictDialog object that creates dialogs
        for adding a new dictionary and changing plugin settings
    */
    DictDialog* dictDialog();

    //! \returns new, clean copy of plugin with settings set as in Settings*
    CommonDictInterface* getNew(const Settings*) const;

    //! \returns whether plugin can start searching
    bool isAvailable() const;

    //! \returns a description of a word given by a QString
    QString search(QString key);

    //! \returns current plugin settings
    Settings* settings();

    //! \returns words count in a dictionary
    long wordsCount();

    //! Sets new settings
    bool setSettings(const Settings*);

    //! \returns plugin icon
    QIcon* icon();

    //! \returns plugin icon's resource path
    QString iconPath();

    /*!
        plugin should delete any files (eg. cache) that have been created and are ready
        to be deleted
    */
    void clean();

    static XdxfDictDownloader dictDownloader;

public Q_SLOTS:
    /*!
        performs search in a dictionary
        \param  word word to search for in a dictionary
        \param limit limit on number of results

        After finishing search it has to emit
        \see CommonDictInterface:finalTranslation  finalTranslation
    */
    QList<Translation*> searchWordList(QString word, int limit=0);

    //! stops current operation
    void stop();

    //! loads translations for each plugin only once
    void retranslate();

Q_SIGNALS:
    //! emitted with percent count of caching progress, and time elapsed from
    //! last signal emit
    void updateCachingProgress(int, int);

private:
    /*!
        \returns true or false depending on whether the dictionary is cached
        or not
    */
    bool isCached();
    /*!
        searches for a list of words similar to a word in a database file
        \param word key compared with keys in a database
        \param limit limits the number of translations in returned list,
        0 means unlimited
        \returns list of translations
    */
    QList<Translation*> searchWordListCache(QString word, int limit=0);

    /*!
        searches for a list of words similar to a word in a xdxf file
        \param word key compared with keys in a xdxf file
        \param limit limits the number of translations in returned list,
        0 means unlimited
        \returns list of translations
    */
    QList<Translation*> searchWordListFile(QString word, int limit=0);

    /*!
        searches for a translation of a word which is exactly like a key
        in a xdxf file
    */
    QString searchFile(QString key);

    /*!
        searches for a translation of a word which is exactly like a key
        in a database file
    */
    QString searchCache(QString key);

    //! scans dictionary file to get information about it
    bool getDictionaryInfo();

    //! counts the keys in a xdxf file
    int countWords();

    /*!
        transforms xdxf files to database files (caching operation)
        \returns true on success, false on failure
    */
    bool makeCache(QString dir);

    static bool dictDownloaderInitialized;

    QString _langFrom;
    QString _langTo;
    QString _name;
    QString _infoNote;
    QString _dictionaryInfo;
    QIcon _icon;
    //! Path to icon
    QString _iconPath;
    QSqlDatabase db;
    QString db_name;
    long _wordsCount;
    volatile bool stopped;
    Settings *_settings;
    XdxfDictDialog* _dictDialog;
    XdxfCachingDialog* cachingDialog;
};

#endif // XDXFPLUGIN_H


